package com.weaver.seconddev.cockpit.cmd;

import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.seconddev.cockpit.util.JucailinCockpitUtils;
import com.weaver.seconddev.jcldoor.util.CommonUtils;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestListInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 流程、日程、协助、文档 4个类型数据
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitWorkflowAnalysisCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitWorkflowAnalysisCmd.class);

    @Autowired
    private JucailinCockpitUtils jucailinCockpitUtils;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;

    @Autowired
    private DatabaseUtils databaseUtils;

    public Map<String, Object> getDataInfo(SimpleEmployee employee, Map<String, Object> paramMap,String tenantKey) {

        Map<String, Object> recordMap = new HashMap<String, Object>();
        try {
            List<Map<String,String>> dataList = getWorkflowDate(employee,tenantKey);
            recordMap.put("leftdata",dataList);
            Map<String,String> chartMap = new HashMap<String,String>();
            chartMap.put("项目管理类流程","16.6");
            chartMap.put("日常办公类流程","16.6");
            chartMap.put("行政事务类流程","40.8");
            chartMap.put("人事类流程","10.0");
            chartMap.put("销售类流程","8.6");
            recordMap.put("rightdata",chartMap);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
        }
        return recordMap;
    }

    /***
     *
     * @return
     */
    public List<Map<String,String>> getWorkflowDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> wflist = new ArrayList<>(100);
        try {
            Map<String,String> wfMap = new HashMap<String,String>();
            String wf_count1 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.TODO_FLOWALL.getTabid());
            String wf_name1 = "待处理流程数量";
            String wf_pkey1 = "workflow.todo.amount";
            String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
            wfMap.put("count",wf_count1);
            wfMap.put("name",wf_name1);
            wfMap.put("url",wf_url1);
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count2 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.MINE_FLOWALL.getTabid());
            String wf_name2 = "我创建的流程";
            String wf_pkey2 = "workflow.mine.amount";
            String wf_url2 =  jucailinCockpitUtils.getUfPropData(wf_pkey2,tenantKey);
            wfMap.put("count",wf_count2);
            wfMap.put("name",wf_name2);
            wfMap.put("url",wf_url2);
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count3 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid());
            String wf_name3 = "我处理的流程";
            String wf_pkey3 = "workflow.done.amount";
            String wf_url3 =  jucailinCockpitUtils.getUfPropData(wf_pkey3,tenantKey);
            wfMap.put("count",wf_count3);
            wfMap.put("name",wf_name3);
            wfMap.put("url",wf_url3);
            wflist.add(wfMap);


            wfMap = new HashMap<String,String>();
            String wf_count4 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.SUPERVISE_ALL.getTabid());
            String wf_name4 = "我督办的流程";
            String wf_pkey4 = "workflow.supervise.amount";
            String wf_url4 =  jucailinCockpitUtils.getUfPropData(wf_pkey4,tenantKey);
            wfMap.put("count",wf_count4);
            wfMap.put("name",wf_name4);
            wfMap.put("url",wf_url4);
            wflist.add(wfMap);


            wfMap = new HashMap<String,String>();
            String wf_count5 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.ATTENTION_ALL.getTabid());
            String wf_name5 = "我关注的流程";
            String wf_pkey5 = "workflow.attention.amount";
            String wf_url5 =  jucailinCockpitUtils.getUfPropData(wf_pkey5,tenantKey);
            wfMap.put("count",wf_count5);
            wfMap.put("name",wf_name5);
            wfMap.put("url",wf_url5);
            wflist.add(wfMap);

            wfMap = getWorkflowDayOperateCount(employee,tenantKey,wf_count3);
            wflist.add(wfMap);

            wfMap = getWorkflowOperateAverageTime(employee,tenantKey);
            wflist.add(wfMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return wflist;
    }

    /**
     *
     * @param employee
     * @return
     */
    public String getWorkflowDataAmount(SimpleEmployee employee,int Type) {
        String count = "0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            WeaResult<RequestCountInfoPAEntity>  requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, Type, conditionEntity);
            int code = requestCountInfo.getCode();
            if(code == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                count = String.valueOf(requestCountInfoPAEntity.getCount());
            }

        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);

        }
        return count;
    }


    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public Map<String,String> getWorkflowOperateAverageTime(SimpleEmployee employee,String tenantKey) {
        String count = "0.0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            WeaResult<List<RequestListInfoPAEntity>> requestListData = wflRequestListRest.getRequestListByTabId(employee, RequestListInitDimensionTabEnum.TODO_FLOWALL.getTabid(), 1, 10, conditionEntity);

            int code = requestListData.getCode();
            if(code == 200){
                Long minutescount = 0L;
                List<RequestListInfoPAEntity> requestCountInfoPAEntityList = requestListData.getData();
                int requestLength = requestCountInfoPAEntityList.size() ;
                for(int i=0;i<requestLength;i++){
                    RequestListInfoPAEntity requestListInfoPAEntity = requestCountInfoPAEntityList.get(i);
                    LocalDateTime receiveDateTime = requestListInfoPAEntity.getReceiveTime();
                    LocalDateTime operateDateTime = requestListInfoPAEntity.getOperateTime();
                    Long minutesDiff = ChronoUnit.MINUTES.between(receiveDateTime, operateDateTime);
                    minutescount += minutesDiff ;
                }
                if(requestLength > 0 && minutescount > 0){
                    double average = minutescount/requestLength ;
                    count = String.valueOf(average);
                }
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);
        }

        Map<String,String> wfMap = new HashMap<String,String>();
        String wf_name1 = "平均流程处理时间";
        String wf_pkey1 = "workflow.time.amount";
        String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
        wfMap.put("count",count);
        wfMap.put("name",wf_name1);
        wfMap.put("url",wf_url1);

        return wfMap;
    }


    public Map<String,String> getWorkflowDayOperateCount(SimpleEmployee employee,String tenantKey,String wf_count3) {
        String count = "0.0";
        String diffdays = "0";
        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";
        log.error("wf_count3:"+wf_count3);
        try{
            long employeeId = employee.getEmployeeId();
            String dataSql =" select a.hiredate,datediff(curdate(),a.hiredate) as diffdays " +
                            " from eteams.employee a   " +
                            " where a.id = ?   " +
                            " and a.tenant_key = ?  " +
                            " and a.status = 'normal'   " +
                            " and a.type = 'inside'";

            log.error("getuserIfo--dataSql:" + dataSql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId + "");
            strlist.add(tenantKey);
            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql, sqlparam);
            List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
            if (CollectionUtils.isNotEmpty(recordList)) {
                Map<String, Object> recordMap = recordList.get(0);
                diffdays = CommonUtils.null2String(recordMap.get("diffdays"));
            }

            log.error("diffdays:"+diffdays);

            if(StringUtils.isNotBlank(wf_count3) && StringUtils.isNotBlank(diffdays)){
                Long dealcount = Long.parseLong(wf_count3);
                Long days = Long.parseLong(diffdays);

                BigDecimal bg = new BigDecimal((double) dealcount/(days*1.00));
                count = String.valueOf(bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);
        }

        Map<String,String> wfMap = new HashMap<String,String>();
        String wf_name1 = "平均每天处理";
        String wf_pkey1 = "workflow.average.amount";
        String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
        wfMap.put("count",count);
        wfMap.put("name",wf_name1);
        wfMap.put("url",wf_url1);

        return wfMap;
    }

}
